######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../data/screen/triggers.4gl  1.13  Delta: 2/21/91
######################################################################

globals "globals.4gl"

######################################################################
function trigger(trig_text)
# returning true/false
######################################################################
#
# Available triggers:
#
# before init
#   - called after defaults are set & before the main program logic
#   - set global variables
#   - change assigned defaults
# before schema
#   - called once per table before any schema work is done on that table
# before load
#   - called once per table before data is loaded into that table
#     from the .unl file (if the -l flag was passed on the command line)
#   - used primarily to set the del_filter variable
#     that defines the rows to delete (if any) before loading data.
#     (default: "1=0" - no delete)
# before unload
#   - called once per table before data is unloaded from that table
#     to the .unl file (if the -u flag was passed on the command line)
#   - used primarily to set the unl_filter variable
#     that defines the rows (from this table) to unload.
#     (default: "1=1" - unload all rows)
#     and to set the data unload order_by variable
#        (default: null - no order)
#        (sample order_by string: "doc_no desc, doc_date")
# before error
#   - called after an sql error has occured, and before the error
#    routines are run.
#  - primarily used to intelligently set the g_err_action variable
#    to either "continue", "quit", or "ask" so the error routines
#    can be invoked accordingly.
#
    define
        n integer,          # generic counter
        c char(1),
        trig_text char(20)

    # Remove data from all tables ?
    if trig_text = "before load"
    then
        # Don't remove data from these tables (or load anything)
        case
        when tab[gc_tab].tabname = "stxaddld"  # usr def flds
          or tab[gc_tab].tabname = "stxaddlr"  # usr def flds ref
          or tab[gc_tab].tabname = "stxacknd"  # acknowledgements
          or tab[gc_tab].tabname = "stxnoted"  # notes
          or tab[gc_tab].tabname = "stxtodod"  # todo
          or tab[gc_tab].tabname = "stxtxtdd"  # default notes
          or tab[gc_tab].tabname = "cgsclipr"  # painter clipboard
          or tab[gc_tab].tabname = "cgsdpndd"  # column dependencies
          or tab[gc_tab].tabname = "cgsifldd"  # input field definitions
          or tab[gc_tab].tabname = "cgsimged"  # screen image
          or tab[gc_tab].tabname = "cgsinptr"  # input area definitions
          or tab[gc_tab].tabname = "cgsscrnr"  # screen definitions
          or tab[gc_tab].tabname = "cgszoomr"  # zoom definitions
          or tab[gc_tab].tabname = "cgxlkupr"  # lookup definitions
          or tab[gc_tab].tabname = "cgxlntod"  # lookup into fields
          or tab[gc_tab].tabname = "cgxmathr"  # math definitions
          or tab[gc_tab].tabname = "cgstrigd"  # trigger storage
          or tab[gc_tab].tabname = "stxuniqc"  # unique key generator
          or tab[gc_tab].tabname = "cgxsorcd"  # source file table
            # Don't delete data, and don't load anything
            let del_filter = "1=0"
            return false

        # Take this opportunity to remove everything from stxfiler
        when tab[gc_tab].tabname = "stxfiler"  # OS input file table
            let del_filter = "1=1"

        # Remove old screen and painter records before loading
        when tab[gc_tab].tabname = "pcdtablr"  # program definition
            let del_filter = "prodid = ", g_quote, "screen", g_quote

        # Error header/detail
        when tab[gc_tab].tabname = "stxerord"  # error detail lines
          or tab[gc_tab].tabname = "stxerorh"  # error header lines
            # Don't delete user defined data
            let del_filter = "userdef is null and (err_module = ",
                g_quote, "lib_scr", g_quote, "  or err_module = ",
                g_quote, "r_menu", g_quote, "  or err_module = ",
                g_quote, "screen", g_quote, "  or err_module = ",
                g_quote, "scr_gen", g_quote, ") and language = ",
                g_quote, "ENG", g_quote

        # Message table
        when tab[gc_tab].tabname = "stxmssgr"  # message lines
            # Don't delete user defined data
            let del_filter = "(mssg_module = ",
                g_quote, "lib_scr", g_quote, "  or mssg_module = ",
                g_quote, "lib_std", g_quote, "  or mssg_module = ",
                g_quote, "lib_ucl", g_quote, "  or mssg_module = ",
                g_quote, "scr_gen", g_quote, ") and language = ",
                g_quote, "ENG", g_quote

        # Help table
        when tab[gc_tab].tabname = "stxhelpd"  # help lines
            # Don't delete user defined data
            let del_filter = "userdef is null and (hlp_module = ",
                g_quote, "lib_scr", g_quote, "  or hlp_module = ",
                g_quote, "screen", g_quote, "  or hlp_module = ",
                g_quote, "zoom_def", g_quote, "  or hlp_module = ",
                g_quote, "scrn_def", g_quote, "  or hlp_module = ",
                g_quote, "painter", g_quote, "  or hlp_module = ",
                g_quote, "ifld_def", g_quote, "  or hlp_module = ",
                g_quote, "math_def", g_quote, "  or hlp_module = ",
                g_quote, "m_file", g_quote, "  or hlp_module = ",
                g_quote, "lkup_def", g_quote, "  or hlp_module = ",
                g_quote, "scr_gen", g_quote, ") and language = ",
                g_quote, "ENG", g_quote

        # Data dictionary tables
        when tab[gc_tab].tabname = "cgdtablr"  # table data
          or tab[gc_tab].tabname = "cgdcolmr"  # column data
            # Delete data from all tables defined
            # in pcdtablr for this module.
            # The .unl files will reload this data.
            let del_filter = "tabname in (",
              "select tabname from pcdtablr where prodid = ",
              g_quote, "screen", g_quote, " or prodid = ", g_quote,
              "stores", g_quote, ")"

        # Load Key definitions only if not already loaded
        when tab[gc_tab].tabname = "stxkeysr"  # keyboard labels
            let del_filter = "1=0"
            select count(*) into n from stxkeysr
            if n > 0
              then return false
              else return true
            end if

        # Only load action definitions if they haven't been already
        when tab[gc_tab].tabname = "stxactnr"  # action definitions
            let del_filter = "1=0"
            select count(*) into n from stxactnr
            if n > 0
              then return false
              else return true
            end if

        # Reload hotkey definitions for scr_gen module
        when tab[gc_tab].tabname = "stxhotkd"
            let del_filter = "(hot_module = ", g_quote, "scr_gen",
                g_quote, " or hot_module is null) and hot_user is null"

        # Reload navigate events for scr_gen module
        when tab[gc_tab].tabname = "stxnvgtd"  # navigate definitions
            let del_filter = "(nav_module = ", g_quote, "scr_gen",
                g_quote, " or nav_module is null) and nav_user is null"

        # Re-load trigger definitions
        when tab[gc_tab].tabname = "cgstrigr"  # trigger definition
            # Delete everything, then reload
            let del_filter = "1=1"

        otherwise
            # Don't clear data if table is unknown
            # (this should never happen)
            let del_filter = "1=0"

        end case
    end if

     if trig_text = "before all indexes"
     then
         call drop_indices("screen")
     end if

    # Update the language variable
    if trig_text = "after schema"
    then
        case
          when tab[gc_tab].tabname = "stxactnr"
            update stxactnr set language = "ENG" where language is null
          when tab[gc_tab].tabname = "stxerord"
            update stxerord set language = "ENG" where language is null
          when tab[gc_tab].tabname = "stxerorh"
            update stxerorh set language = "ENG" where language is null
          when tab[gc_tab].tabname = "stxmssgr"
            update stxmssgr set language = "ENG" where language is null
          when tab[gc_tab].tabname = "stxhelpd"
            update stxhelpd set language = "ENG" where language is null
          when tab[gc_tab].tabname = "cgxlntod"
            update cgxlntod set lkup_from = lkup_into where 1=1
        end case
    end if

    return true
end function
# trigger()

#####################################################################
function drop_indices(prod_id)
#####################################################################
#  This function is used for dropping all indices pertaining to the
#  this moudules particular tables. It is a function we will use for
#  this 3.5 release only. Its main purpose is for maintainence.

    define
        prod_id      char(10),
        scratch      char(250),
        index_name   char(20),
        c            char(1)

    let scratch = "select idxname from sysindexes, systables, ",
        "pcdtablr where sysindexes.tabid = systables.tabid and ",
        "systables.tabname = pcdtablr.tabname and ",
        "pcdtablr.prodid = ", g_quote, prod_id, g_quote

    prepare drop_loop from scratch
    declare drop_cursor cursor for drop_loop

    foreach drop_cursor into index_name
        let scratch = "drop index ", index_name clipped
        prepare drop_index from scratch
        execute drop_index
    end foreach

end function
# drop_indices()
