#####################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../data/stores/triggers.4gl  1.1  Delta: 11/23/90
#####################################################################

globals "globals.4gl"

#####################################################################
function trigger(trig_text)
# returning true/false
#####################################################################
#
# Available triggers:
#
# before init
#   - called after defaults are set & before the main program logic
#   - set global variables
#   - change assigned defaults
# before schema
#   - called once per table before any schema work is done on that table
# before load
#   - called once per table before data is loaded into that table
#     from the .unl file (if the -l flag was passed on the command line)
#   - used primarily to set the del_filter variable
#     that defines the rows to delete (if any) before loading data.
#     (default: "1=0" - no delete)
# before unload
#   - called once per table before data is unloaded from that table
#     to the .unl file (if the -u flag was passed on the command line)
#   - used primarily to set the unl_filter variable
#     that defines the rows (from this table) to unload.
#     (default: "1=1" - unload all rows)
#     and to set the data unload order_by variable
#        (default: null - no order)
#        (sample order_by string: "doc_no desc, doc_date")
# before error
#   - called after an sql error has occured, and before the error
#    routines are run.
#  - primarily used to intelligently set the g_err_action variable
#    to either "continue", "quit", or "ask" so the error routines
#    can be invoked accordingly.
#
    define
        trig_text char(20)

    # Remove data from tables (always a fresh install - demo db)
    if trig_text = "before load"
    then
        case
          when tab[gc_tab].tabname = "pcdtablr"  # program definition
            let del_filter = "prodid = \"screen\" ",
             "or prodid = \"stores\" "

          otherwise
            # Clear old data before loading
            let del_filter = "1=1"
        end case
    end if

    return true
end function
# trigger()

