######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../scr.4gs/zoom.4gl  1.5  Delta: 3/27/91
######################################################################

globals "globals.4gl"

define
    arr_mesgs array[1] of record    # Message text
        mssg_text char(132)
    end record,
    mssg_prep char(1),              # Y/null Message cursor prepared?

    zoom_cnt smallint    # number of rows in the zoom array


######################################################################
function zoom(scrname, entry_filter)
# Returning true if the zoom returned anything
######################################################################
# This function bootstraps a zoom screen.  It first remembers a few
# global and library variables, then it registers the zoom screen
# with the screen manager.  Upon return, it resets the global
# and local variables to their state before the zoom call.
#
    define
        scrname char(7),         # Screen name to register
        entry_filter char(512),  # Zoom screen entry filter
        o_sql_filter char(512)   # old sql_filter

    # Trap fatal errors
    whenever error call error_handler

    # Remember old sql filter
    let o_sql_filter = sql_filter

    # Place the filter into scratch
    # (the order is set upon "init" in scr_register)
    if entry_filter = "(see scratch)"
      then let sql_filter = scratch
      else let sql_filter = entry_filter
    end if

    # Register the zoom screen with the screen manager
    call scr_register(scrname, "zoom")

    # Restore the sql filter
    let sql_filter = o_sql_filter

    # Did the zoom return anything?
    if scratch is not null
      then return true
      else return false
    end if

end function
# zoom()

######################################################################
function auto_zoom(scrname, zoomtab, entry_filter)
# returning true/false (true if zoomed)
######################################################################
# This function scans the data in the field of entry, and if it
# contains an asterisk (*), then it calls the zoom screen, passing
# the contents of the field as a "matches" type filter.
# If a zoom screen is entered, the function returns true.  If the
# user pressed [DEL] during zoom (zoom returned false), then
# scratch will be filled with prev_data.
# If there was no asterisk (no auto zoom) then it returns false.
#
    define
        scrname char(7),         # Zoom Screen name
        zoomtab char(37),        # Main zoom table name
        zoomtab_chk char(37),    # clipped zoomtab to speed parsing  
        entry_filter char(512),  # Zoom screen entry filter
        thiscol char(18),        # Current column name
        quote char(1),           # storage for the quote character (")
        n smallint,              # generic counter
        m smallint,              # generic counter
        z smallint,              # generic counter
        x smallint               # generic counter

    let quote = "\""

    # Set the 'this_data' variable
    call switchbox("set this_data")
    
    let zoomtab_chk = zoomtab clipped

    # Check for an asterisk
    for n = 1 to length(this_data)
        if this_data[n,n] = "*"
        then
                let z = length(zoomtab_chk)
                for x = z to 1 step -1  
                    if zoomtab_chk[x] = "."
                    then 
                         let thiscol = zoomtab_chk[x+1,z]
                         let zoomtab = zoomtab_chk[1,x-1]
                         exit for
                    end if
                end for
                if thiscol is null 
                then
                # Determine this column name from scr_fld
                     let thiscol = scr_fld  # If not in tab.col format
                     let m = length(scr_fld)
                     for n = 1 to m - 1
                         if scr_fld[n,n] = "."
                         then
                             let thiscol = scr_fld[n + 1,m]
                              exit for
                         end if
                      end for
                 end if

            # Place the filter into scratch
            if entry_filter = "(see scratch)"
            then else
                let scratch = entry_filter
            end if

            # Does it need an 'and'?
            if scratch is not null
              then let scratch = scratch clipped, " and"
            end if

            # Add the matches clause
            let scratch = scratch clipped, " ", zoomtab clipped,
              ".", thiscol clipped, " matches ",
              quote, this_data clipped, quote

            # Zoom into this screen
            if zoom(scrname,"(see scratch)") = false
              then let scratch = prev_data
            end if
            return true

        end if  # asterisk found
    end for     # all characters in this_data

    # no auto zoom
    return false

end function
# auto_zoom()

######################################################################
function fcm_zoom()
######################################################################
# This is the upper level flow control manager for zoom screens.
#
    call menu_line()

    if mssg_prep is null then
       let arr_mesgs[1].mssg_text = fg_message("lib_scr","zoom",1)
#1: " rows selected)"
       let mssg_prep = "Y"
    end if

    # Build the filter if the screen needs it
    if sql_filter is null
    then
        call lib_message("find")
        call switchbox("construct")
        if int_flag != 0 then return end if
        let sql_filter = scratch
    end if

    # Dynamically determine the window size
    call window_size()

    # Go to the zoom screen
    if do_zoom(true) = false then return end if

# the following is the language independant menu routine in menuzoom.4gl
# which is set up to allow different language menus to be utilized

    call zoom_menu()

    # Set the current element to 0 if interrupted
    if int_flag != 0 then let p_cur = 0 end if

end function
# fcm_zoom()

######################################################################
function do_zoom(read_too)
# returning false if user presses interrupt in zoom
######################################################################
# This function reads the rows into the zoom array, then calls
# display array.
#
    define
        read_too smallint  # (boolean) Perform a disk read also?

    let int_flag = 0
    if read_too
    then
        call ring_clear()            
        call please_wait()           # Ask them to wait upon disk read
        call lib_filtord()           # Default sql_filter and sql_order
        call switchbox("read")       # Read in the rows
        let zoom_cnt = scratch       # Number of elements in array
    end if

    # Display the zoom message
    call lib_message("zoom")

    # Display the number of rows selected
    let scratch = "(", zoom_cnt using "<<<<&",
                  arr_mesgs[1].mssg_text clipped
    call lib_message("scr_bottom")

    # If no rows to display, return to the zoom menu
    if zoom_cnt = 0 then let p_cur = 0 return true end if

    # Enter display array
    call set_count(zoom_cnt)
    call switchbox("display array") 

    # Set the current element
    let p_cur = arr_curr()

    # Set the current element to 0 if interrupted
    if int_flag != 0 then let p_cur = 0 end if

    # Exit the zoom screen
    if scratch = "quit" or int_flag != 0
      then return false
      else return true
    end if
end function
# do_zoom()
