#####################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../user_nul.4gs/lib_eror.4gl  1.2  Delta: 1/25/91
#####################################################################

globals "globals.4gl"

define
    err_lang char(3),                # error language
    err_prep char(1),                # Y/null Error cursor prepared?
    err_mod  char(8),                # error module
    err_prog char(8),                # error program
    err_num  smallint,               # error number
    one_liner char(60),              # one line error message
    tech_msg char(100),              # technical description of error
    errstat integer                  # status variable upon entry

######################################################################
function lib_error(err_module, err_program, err_number, tech_message)
######################################################################
# This is the fourgen programmer/user definable error routine.
#
    define
        err_module  char(8),    # module name
        err_program char(8),    # program name within the module
        err_number  smallint,   # error number within the program
        tech_message char(100), # technical error message
        c           char(1)     # entry char

    # Set the static variables
    let errstat = status
    if errstat = 0 then let errstat = sqlca.sqlcode end if
    let err_mod = err_module
    let err_prog = err_program
    let err_num = err_number
    if tech_message = "(see scratch)"
      then let tech_msg = scratch
      else let tech_msg = tech_message
    end if

    # If the error number is 0, then the user selected control-e
    # during input.  Allow them to choose the error to edit.
    if err_num = 0
    then
        call er_choose()
        return
    end if

    whenever error continue
    open window errwin at 22, 8 with 2 rows, 66 columns
        attribute (border, white)
    whenever error call error_handler

    display " Error:" at 1,1

    if one_liner is null
      then call er_read(err_mod, err_prog, err_num)
    end if

    display " ",one_liner clipped," " at 1,10 attribute(reverse)

    if fg_prompt("",
      "Press [ENTER] to continue")
    then end if

    close window errwin

    # Zoom selected
    if scr_funct = "zoom"
    then
        call er_log()
        call er_info()
    end if

    # Reset error condition
    let err_mod = ""
    let err_prog= ""
    let err_num = 0
    let one_liner = ""
    let tech_msg = ""
    return
end function
# lib_error()

#######################################################################
function er_info()
#######################################################################
# This function permits viewing and updating of the error text.
#
call ucl_text("er_info") call textview("Expanded Error")
end function
# er_info()

#####################################################################
function er_read(err_mod, err_prog, err_num)
#####################################################################
# This function reads the error text from the user defined table,
# or from the system defined table if no data exists in the user
# defined error text table.
#
    define
        err_mod  char(8),                # error module (passed)
        err_prog char(8),                # error program (passed)
        err_num  smallint                # error number (passed)

    # Prepare the cursors upon first call
    if err_prep is null
    then
        let scratch = "select err_line, userdef from stxerorh",
          " where err_module = ?",
          " and err_program = ?",
          " and err_number = ?",
          " and language = \"", get_scrlib("language"), "\"",
          " order by userdef desc"
        prepare onlspec from scratch
        declare s_erorh cursor for onlspec

        let err_prep = "Y"
    end if

    # Fetch the oneliner
    open s_erorh using err_mod, err_prog, err_num
    fetch s_erorh into one_liner

    # Check for notfound condition
    if sqlca.sqlcode = notfound
    then
        # Try to read this new error condition from the error file
        if    err_mod = "lib_scr"
          and err_prog = "error"
          and err_num = 1
        then
            # loop situation
            let one_liner = "Unknown - Cannot locate error text."
            return
        end if
        call er_read("lib_scr","error",1)
        return
    end if

    # No use leaving the cursors open
    close s_erorh

end function
# er_read()

######################################################################
function err_line(errorline)
######################################################################
# This function allows you to programatically change the 'one liner'
# error message.  This is useful for standard errors (like lookups)
# that you would like different text for the one liner.
#
    define
        errorline char(60)

    let one_liner = errorline
end function
# err_line

#####################################################################
function er_edit(heading)
#####################################################################
# This function allows the user to type in additional information
# that will be logged with this error.
#
    define
        heading char(30)  # Heading for the editor

    if textedit(heading,60)
    then
        while textget()
            call errorlog(scratch[1,60])
        end while
    end if

end function
# er_edit()

#####################################################################
function er_status()
#####################################################################
# This function displays the status of the error onto the screen.
#
    define
        c char(1),  # temporary input char
        n smallint  # temporary counter

    whenever error continue
    let status = 0
    open window statuswin at 4, 8 with 18 rows, 66 columns
        attribute (border, white)
    whenever error call error_handler
    if status = -1143
    then
        call scr_error("recursive","statuswin")
        return
    end if
    call menu_line()
    display "Technical status screen" at 1,2
    let scratch = get_scrlib("version")
    display "Code generator version: ", scratch clipped at 4,2
    let scratch = get_scrlib("dbname")
    display "Database name: ", scratch clipped at 4,33
    if progid is not null
      then display "Program   ID: ", progid at 6,2
    end if
    if scr_id is not null
      then display "Screen    ID: ", scr_id at 7,2
    end if
    if scr_fld is not null
      then display "Field: ", scr_fld at 7,26
    end if
    if err_mod is not null
    then
        display "Error module: ", err_mod at 8,2
        display "Error program: ", err_prog at 8,26
        display "Error no.: ", err_num using "<<<<<" at 8,51
        display "Error line  : ", one_liner at 9,2
    end if
    if tech_msg is not null
    then
        display "Technical message: " at 12,2
        let scratch = tech_msg[1,65]
        display scratch clipped at 13,2
        let scratch = tech_msg[66,100]
        display scratch clipped at 14,2
    end if
    display "Status variable: ", errstat using "-<<<&" at 15,2
    if errstat < 0
    then
        let scratch = err_get(errstat)
        display scratch at 16,2
    end if
    prompt " Press [ENTER] to continue: " for char c
    close window statuswin
    let int_flag = 0
end function
# er_status()

#####################################################################
function er_log()
#####################################################################
# This function logs the status of the error to the logfile
#
    define
        nl char(1),  # storage for the newline character
        c char(1),   # temporary character
        n smallint   # temporary

    let nl = "\n"
    let scratch =
      "==========================================", nl,
      "Error status log. Requested by:  ", fg_username(), nl,
      "Codegen  version: ", get_scrlib("version"), nl,
      "Database    name: ", get_scrlib("dbname"), nl,
      "Program       ID: ", progid
    call errorlog(scratch[1,512])
    let scratch =
      "Screen        ID: ", scr_id clipped, nl,
      "Screen     field: ", scr_fld clipped, nl,
      "Error     module: ", err_mod clipped, nl,
      "Error    program: ", err_prog clipped, nl,
      "Error     number: ", err_num using "<<<", nl, 
      "SQL error status: ", errstat using "-<<<&"
    call errorlog(scratch[1,512])
    if tech_msg is not null
    then
        let scratch = "Technical   note:", nl, tech_msg
        call errorlog(scratch[1,512])
    end if
    if errstat < 0
    then
        let scratch = "SQL  error  text:", nl, err_get(errstat)
        call errorlog(scratch[1,512])
    end if

    let scratch =
      "End of status log", nl,
      "==========================================", nl
    call errorlog(scratch[1,512])

end function
# er_log()
  
function er_display() end function
function er_update() end function
function er_getnum() end function

function er_choose()
call ucl_text("er_chose") call textview("Error Edit")
end function
function er_mn_display() end function
